/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.birds;

import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.taxonomy.TaxonomyManager;
import it.tidalwave.bluebill.taxonomy.io.Exporter;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.logging.Logger;
import java.util.logging.Handler;
import java.util.logging.LogManager;
import it.tidalwave.util.logging.SingleLineLogFormatter;
import org.openrdf.elmo.ElmoManager;
import org.openrdf.elmo.ElmoModule;
import org.openrdf.elmo.sesame.SesameManagerFactory;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.rio.RDFParseException;
import org.openrdf.sail.memory.MemoryStore;
import it.tidalwave.openrdf.elmo.ElmoManagerThreadLocal;
import java.io.InputStreamReader;
import java.io.Reader;
import org.openrdf.rio.RDFFormat;
import javax.annotation.Nonnull;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.zip.GZIPInputStream;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public abstract class GeneratorSupport
  {
    private static final String CLASS = GeneratorSupport.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    protected final File resourcesFolder;

    protected final File targetFolder;

    protected SesameManagerFactory smf;

    protected Repository repository;

    @Nonnull
    protected final String taxonomyName;

    @Nonnull
    protected final String taxonomyFileName;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public GeneratorSupport (final @Nonnull String baseDir,
                             final @Nonnull String taxonomyName,
                             final @Nonnull String taxonomyFileName)
      {
        this.taxonomyName = taxonomyName;
        this.taxonomyFileName = taxonomyFileName;
        resourcesFolder = new File(baseDir + "/target/");
        targetFolder = new File(baseDir + "/target/test-artifacts/");
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void execute()
      throws Exception
      {
        setupLogging(GeneratorSupport.class);
        final File rdfFile = new File(targetFolder, taxonomyFileName + ".rdf");
        final File n3File = new File(targetFolder, taxonomyFileName + ".n3");
        //
        // This allows to run this from 'mvn deploy' without regenerating the stuff in case they have already been done.
        //
        if (rdfFile.exists() || n3File.exists())
          {
            System.err.printf("%s or %s exist: won't run. Please delete both of them and re-run me.\n", rdfFile, n3File);
          }

        else
          {
            initialize();
            final BirdTaxonomyImporter importer = run();

            final File file = new File(targetFolder, taxonomyFileName + "-importer.ser");
            final ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(file));
            oos.writeObject(importer);
            oos.close();
        
            final Taxonomy taxonomy = Locator.find(TaxonomyManager.class).findTaxonomyByName(taxonomyName, repository);
            taxonomy.as(Exporter.class).export(rdfFile);
            taxonomy.as(Exporter.class).export(n3File);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public abstract BirdTaxonomyImporter run()
      throws Exception;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void initialize()
      throws Exception
      {
        repository = new SailRepository(new MemoryStore());
        repository.initialize();
        final ElmoModule elmoModule = new ElmoModule();
        smf = new SesameManagerFactory(elmoModule, repository);
        final ElmoManager em = smf.createElmoManager();
        ElmoManagerThreadLocal.set(em);
        targetFolder.mkdirs();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void close()
      {
        ElmoManagerThreadLocal.get().close();
        ElmoManagerThreadLocal.set(null);
        smf.close();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static void setupLogging (final @Nonnull Class<?> clazz)
      {
        try
          {
            new File("target/logs").mkdirs();
            final InputStream is = clazz.getResourceAsStream("log.properties");

            if (is == null)
              {
                System.err.println("NO LOGGING CONFIGURATION");
              }
            else
              {
                LogManager.getLogManager().readConfiguration(is);
                is.close();
              }
            //
            // The formatter must be set programmatically as the property in the log.properties won't
            // be honored. I suspect it is related with NetBeans module classloaders as the formatter
            // is loaded inside LogManager by using the SystemClassLoader, which only sees the classpath.
            //
            final SingleLineLogFormatter formatter = new SingleLineLogFormatter();
            java.util.logging.Logger rootLogger = java.util.logging.Logger.getLogger(clazz.getName());

            while (rootLogger.getParent() != null)
              {
                rootLogger = rootLogger.getParent();
              }

            for (final Handler handler : rootLogger.getHandlers())
              {
                handler.setFormatter(formatter);
              }
          }
        catch (Exception e)
          {
            e.printStackTrace();
          }
      }
  }
